"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = require("aws-sdk");
class Bucket {
    constructor(name) {
        this.name = name;
        this.requestProperties = { Bucket: this.name };
        this.s3 = new aws_sdk_1.S3();
    }
    async allowsPublicAccessViaACL() {
        return new Promise((resolve, reject) => {
            this.s3.getBucketAcl(this.requestProperties, (error, data) => {
                if (error !== null) {
                    return reject(error);
                }
                if (data === null || !Array.isArray(data.Grants) || data.Grants.length === 0) {
                    return resolve(false);
                }
                for (let grant of data.Grants) {
                    if (grant.Grantee === undefined || grant.Grantee.Type !== 'Group' || grant.Grantee.URI === undefined) {
                        continue;
                    }
                    // tslint:disable-next-line:no-http-string
                    if (['http://acs.amazonaws.com/groups/global/AllUsers', 'http://acs.amazonaws.com/groups/global/AuthenticatedUsers'].includes(grant.Grantee.URI)) {
                        return resolve(true);
                    }
                }
                resolve(false);
            });
        });
    }
    async getCloudFrontOriginAccessIdentities() {
        const cloudfrontPrefix = 'arn:aws:iam::cloudfront:user/CloudFront Origin Access Identity ';
        return new Promise((resolve, reject) => {
            this.s3.getBucketPolicy(this.requestProperties, (error, data) => {
                if (error !== null && error.code !== 'NoSuchBucketPolicy') {
                    return reject(error);
                }
                if (data === null || data.Policy === undefined) {
                    return resolve([]);
                }
                const policy = JSON.parse(data.Policy);
                const identities = [];
                for (let statement of policy.Statement) {
                    if (statement.Principal === undefined || statement.Principal.AWS === undefined) {
                        continue;
                    }
                    const principals = Array.isArray(statement.Principal.AWS) ?
                        statement.Principal.AWS
                        : [statement.Principal.AWS];
                    for (let principal of principals) {
                        if (principal.indexOf(cloudfrontPrefix) === 0) {
                            const identity = principal.replace(cloudfrontPrefix, '');
                            identities.push(identity);
                        }
                    }
                }
                resolve(identities);
            });
        });
    }
    async getPublicAccessConfiguration() {
        return new Promise((resolve, reject) => {
            if (this.publicAccessConfiguration !== undefined) {
                return resolve(this.publicAccessConfiguration);
            }
            this.s3.getPublicAccessBlock(this.requestProperties, (error, data) => {
                if (error !== null && error.code !== 'NoSuchPublicAccessBlockConfiguration') {
                    return reject(error);
                }
                const defaultConfig = {
                    BlockPublicAcls: false,
                    BlockPublicPolicy: false,
                    RestrictPublicBuckets: false,
                    IgnorePublicAcls: false
                };
                this.publicAccessConfiguration = (data === null) ? defaultConfig
                    : Object.assign(defaultConfig, data.PublicAccessBlockConfiguration);
                resolve(this.publicAccessConfiguration);
            });
        });
    }
    async getPolicyWildcardEntities() {
        return new Promise((resolve, reject) => {
            this.s3.getBucketPolicy(this.requestProperties, (error, data) => {
                if (error !== null && error.code !== 'NoSuchBucketPolicy') {
                    return reject(error);
                }
                if (data === null || data.Policy === undefined) {
                    return resolve([]);
                }
                const policy = JSON.parse(data.Policy);
                const statements = [];
                for (let statement of policy.Statement) {
                    if (statement.Effect === 'Deny') {
                        continue;
                    }
                    if (statement.Principal === '*' || statement.Principal.AWS === '*') {
                        statements.push(statement);
                        continue;
                    }
                    if (Array.isArray(statement.Principal.AWS)) {
                        const index = statement.Principal.AWS.indexOf('*');
                        if (index > -1) {
                            statements.push(statement.Principal.AWS[index]);
                        }
                    }
                    if (statement.Action === '*') {
                        statements.push(statement);
                        continue;
                    }
                    if (Array.isArray(statement.Action)) {
                        const index = statement.Action.indexOf('*');
                        if (index > -1) {
                            statements.push(statement.Action[index]);
                        }
                    }
                }
                resolve(statements);
            });
        });
    }
    async getSSEAlgorithm() {
        return new Promise((resolve, reject) => {
            this.s3.getBucketEncryption(this.requestProperties, (error, data) => {
                if (error !== null && error.code !== 'ServerSideEncryptionConfigurationNotFoundError') {
                    return reject(error);
                }
                if (data === null || data.ServerSideEncryptionConfiguration === undefined || data.ServerSideEncryptionConfiguration.Rules[0].ApplyServerSideEncryptionByDefault === undefined) {
                    return resolve(null);
                }
                const algorithm = data.ServerSideEncryptionConfiguration.Rules[0].ApplyServerSideEncryptionByDefault.SSEAlgorithm;
                resolve(algorithm);
            });
        });
    }
    async hasMFADeleteEnabled() {
        return new Promise((resolve, reject) => {
            this.getBucketVersioning()
                .then((data) => {
                if (data === null || data.MFADelete !== 'Enabled') {
                    return resolve(false);
                }
                resolve(true);
            })
                .catch((error) => {
                reject(error);
            });
        });
    }
    async hasStaticWebsiteHosting() {
        return new Promise((resolve, reject) => {
            this.s3.getBucketWebsite(this.requestProperties, (error, data) => {
                if (error !== null && error.code !== 'NoSuchWebsiteConfiguration') {
                    return reject(error);
                }
                if (data === null) {
                    return resolve(false);
                }
                resolve(true);
            });
        });
    }
    async hasVersioningEnabled() {
        return new Promise((resolve, reject) => {
            this.getBucketVersioning()
                .then((data) => {
                if (data === null || data.Status !== 'Enabled') {
                    return resolve(false);
                }
                resolve(true);
            })
                .catch((error) => {
                reject(error);
            });
        });
    }
    async getLoggingTargetBucket() {
        return new Promise((resolve, reject) => {
            this.s3.getBucketLogging(this.requestProperties, (error, data) => {
                if (error !== null) {
                    return reject(error);
                }
                if (data === null || data.LoggingEnabled === undefined) {
                    return resolve(null);
                }
                if (data.LoggingEnabled.TargetBucket) {
                    return resolve(data.LoggingEnabled.TargetBucket);
                }
                resolve(null);
            });
        });
    }
    async getBucketVersioning() {
        return new Promise((resolve, reject) => {
            if (this.bucketVersioning !== undefined) {
                return resolve(this.bucketVersioning);
            }
            this.s3.getBucketVersioning(this.requestProperties, (error, data) => {
                if (error !== null) {
                    return reject(error);
                }
                this.bucketVersioning = data;
                resolve(data);
            });
        });
    }
}
exports.default = Bucket;
