"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Listr = require('listr');
class Console {
    constructor() {
        this.listrOptions = {
            exitOnError: false,
            collapse: false,
            concurrent: true
        };
    }
    run(buckets) {
        const tasks = new Listr([], this.listrOptions);
        buckets.forEach(bucket => {
            tasks.add([
                {
                    title: bucket.name,
                    task: () => {
                        return new Listr([
                            {
                                title: 'Bucket public access configuration',
                                task: () => new Listr([
                                    {
                                        title: 'BlockPublicAcls',
                                        task: (context, task) => this.checkPublicAccesBlockFor(task, bucket, 'BlockPublicAcls')
                                    },
                                    {
                                        title: 'IgnorePublicAcls',
                                        task: (context, task) => this.checkPublicAccesBlockFor(task, bucket, 'IgnorePublicAcls')
                                    },
                                    {
                                        title: 'BlockPublicPolicy',
                                        task: (context, task) => this.checkPublicAccesBlockFor(task, bucket, 'BlockPublicPolicy')
                                    },
                                    {
                                        title: 'RestrictPublicBuckets',
                                        task: (context, task) => this.checkPublicAccesBlockFor(task, bucket, 'RestrictPublicBuckets')
                                    }
                                ], this.listrOptions)
                            },
                            {
                                title: 'Server side encryption is enabled',
                                task: (context, task) => this.checkBucketEncryption(task, bucket)
                            },
                            {
                                title: 'Object versioning is enabled',
                                task: (context, task) => this.checkBucketVersioning(task, bucket)
                            },
                            {
                                title: 'MFA Delete is enabled',
                                task: (context, task) => this.checkMFADelete(task, bucket)
                            },
                            {
                                title: 'Static website hosting is disabled',
                                task: (context, task) => this.checkBucketWebsite(task, bucket)
                            },
                            {
                                title: 'Bucket policy doesn\'t allow a wildcard entity',
                                task: (context, task) => this.checkThatBucketPolicyDoesntAllowWildcardEntity(task, bucket)
                            },
                            {
                                title: 'Bucket ACL doesn\'t allow access to "Everyone" or "Any authenticated AWS user"',
                                task: (context, task) => this.checkBucketAcl(task, bucket)
                            },
                            {
                                title: 'Logging is enabled',
                                task: (context, task) => this.checkBucketLogging(task, bucket)
                            },
                            {
                                title: 'CloudFront Distributions',
                                task: (context, task) => this.checkCloudFrontDistributions(task, bucket)
                            }
                        ], this.listrOptions);
                    }
                }
            ]);
        });
        new Listr([
            {
                title: `Checking ${buckets.length} bucket${buckets.length === 1 ? '' : 's'}`,
                task: () => tasks
            }
        ], this.listrOptions).run().catch((err) => { });
    }
    async checkPublicAccesBlockFor(task, bucket, setting) {
        const publicAccessBlockConfiguration = await bucket.getPublicAccessConfiguration()
            .catch((error) => {
            task.skip(error.message);
        });
        if (publicAccessBlockConfiguration !== undefined && publicAccessBlockConfiguration[setting] === false) {
            task.title = `${setting} is set to false`;
            throw new Error();
        }
    }
    async checkBucketEncryption(task, bucket) {
        return bucket.getSSEAlgorithm()
            .catch((error) => {
            task.skip(error.message);
        })
            .then((algorithm) => {
            if (algorithm === null) {
                task.title = 'Server side encryption is not enabled';
                throw new Error();
            }
            task.output = `Encryption algorithm is ${algorithm}`;
        });
    }
    async checkBucketLogging(task, bucket) {
        const targetBucket = await bucket.getLoggingTargetBucket()
            .catch((error) => {
            task.skip(error.message);
        });
        if (targetBucket === null) {
            task.title = 'Logging is not enabled';
            throw new Error();
        }
        task.output = `Logging to ${targetBucket}`;
    }
    async checkBucketVersioning(task, bucket) {
        const isEnabled = await bucket.hasVersioningEnabled()
            .catch((error) => {
            task.skip(error.message);
        });
        if (isEnabled === false) {
            task.title = 'Object versioning is not enabled';
            throw new Error();
        }
    }
    async checkBucketWebsite(task, bucket) {
        const isEnabled = await bucket.hasStaticWebsiteHosting()
            .catch((error) => {
            task.skip(error.message);
        });
        if (isEnabled === true) {
            task.title = 'Static website hosting is enabled';
            throw new Error();
        }
    }
    async checkCloudFrontDistributions(task, bucket) {
        bucket.getCloudFrontOriginAccessIdentities()
            .catch((error) => {
            task.skip(error.message);
        })
            .then((identities) => {
            if (identities === undefined) {
                return;
            }
            if (identities.length === 0) {
                task.title = 'Bucket is not associated with any CloudFront distributions';
                return;
            }
            task.title = `Bucket is associated with ${identities.length} CloudFront distribution${identities.length === 1 ? '' : 's'}`;
        });
    }
    async checkThatBucketPolicyDoesntAllowWildcardEntity(task, bucket) {
        const statements = await bucket.getPolicyWildcardEntities()
            .catch((error) => {
            task.skip(error.message);
        });
        if (Array.isArray(statements) && statements.length > 0) {
            task.title = 'Bucket policy contains wildcard entities';
            task.output = `Bucket has ${statements.length} statement${statements.length === 1 ? '' : 's'} with wildcard entities`;
            throw new Error();
        }
    }
    async checkBucketAcl(task, bucket) {
        const allowsPublicAccess = await bucket.allowsPublicAccessViaACL()
            .catch((error) => {
            task.skip(error.message);
        });
        if (allowsPublicAccess === true) {
            task.title = 'Bucket allows public access via ACL';
            throw new Error();
        }
    }
    async checkMFADelete(task, bucket) {
        const isEnabled = await bucket.hasMFADeleteEnabled()
            .catch((error) => {
            task.skip(error.message);
        });
        if (isEnabled === false) {
            task.title = 'MFA Delete is not enabled';
            throw new Error();
        }
    }
}
exports.default = Console;
