"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Csv {
    constructor(output) {
        this.fields = {
            name: 'Bucket name',
            BlockPublicAcls: 'BlockPublicAcls is enabled',
            BlockPublicPolicy: 'BlockPublicPolicy is enabled',
            RestrictPublicBuckets: 'RestrictPublicBuckets is enabled',
            IgnorePublicAcls: 'IgnorePublicAcls is enabled',
            sse: 'Server side encryption algorith',
            versioning: 'Object versioning is enabled',
            mfa_delete: 'MFA Delete is enabled',
            static_website: 'Static website hosting is disabled',
            bucket_policy: 'Bucket policy doesn\'t allow a wildcard entity',
            bucket_acl: 'Bucket ACL doesn\'t allow public access',
            logging: 'Bucket logging target bucket',
            cloudfront: 'CloudFront Origin Access Identities'
        };
        this.output = output;
    }
    async run(buckets) {
        this.output(Object.values(this.fields).join(','));
        buckets.forEach(async (bucket) => {
            const bucketDetails = { name: bucket.name };
            Promise.all([
                this.populatePublicAccessConfiguration(bucket, bucketDetails),
                this.populateSSEField(bucket, bucketDetails),
                this.populateLoggingField(bucket, bucketDetails),
                this.populateVersioningField(bucket, bucketDetails),
                this.populateStaticWebsiteField(bucket, bucketDetails),
                this.populatePolicyField(bucket, bucketDetails),
                this.populateACLField(bucket, bucketDetails),
                this.populateMFADeleteField(bucket, bucketDetails),
                this.populateCloudfrontField(bucket, bucketDetails)
            ]).then(() => {
                const output = [];
                Object.keys(this.fields).forEach((field) => {
                    output.push(bucketDetails[field]);
                });
                this.output(output.join(','));
            });
        });
    }
    populatePublicAccessConfiguration(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.getPublicAccessConfiguration()
                .then((bucketPublicAccessBlock) => {
                Object.assign(bucketDetails, bucketPublicAccessBlock);
            })
                .catch((error) => {
                Object.assign(bucketDetails, {
                    BlockPublicAcls: error.message,
                    BlockPublicPolicy: error.message,
                    RestrictPublicBuckets: error.message,
                    IgnorePublicAcls: error.message
                });
            })
                .finally(() => resolve());
        });
    }
    populateSSEField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.getSSEAlgorithm()
                .then((algorithm) => {
                bucketDetails.sse = algorithm || 'Disabled';
            })
                .catch((error) => {
                bucketDetails.sse = error.message;
            })
                .finally(() => resolve());
        });
    }
    populateLoggingField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.getLoggingTargetBucket()
                .then((bucket) => {
                bucketDetails.logging = bucket || 'Disabled';
            })
                .catch((error) => {
                bucketDetails.logging = error.message;
            })
                .finally(() => resolve());
        });
    }
    populateVersioningField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.hasVersioningEnabled()
                .then((isEnabled) => {
                bucketDetails.versioning = isEnabled;
            })
                .catch((error) => {
                bucketDetails.versioning = error.message;
            })
                .finally(() => resolve());
        });
    }
    populateStaticWebsiteField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.hasStaticWebsiteHosting()
                .then((isEnabled) => {
                bucketDetails.static_website = isEnabled;
            })
                .catch((error) => {
                bucketDetails.static_website = error.message;
            })
                .finally(() => resolve());
        });
    }
    populatePolicyField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.getPolicyWildcardEntities()
                .then((entities) => {
                bucketDetails.bucket_policy = entities.length === 0;
            })
                .catch((error) => {
                bucketDetails.bucket_policy = error.message;
            })
                .finally(() => resolve());
        });
    }
    populateACLField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.allowsPublicAccessViaACL()
                .then((isPublic) => {
                bucketDetails.bucket_acl = isPublic === false;
            })
                .catch((error) => {
                bucketDetails.bucket_acl = error.message;
            })
                .finally(() => resolve());
        });
    }
    populateMFADeleteField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.hasMFADeleteEnabled()
                .then((isEnabled) => {
                bucketDetails.mfa_delete = isEnabled;
            })
                .catch((error) => {
                bucketDetails.mfa_delete = error.message;
            })
                .finally(() => resolve());
        });
    }
    populateCloudfrontField(bucket, bucketDetails) {
        return new Promise(resolve => {
            bucket.getCloudFrontOriginAccessIdentities()
                .then((identities) => {
                bucketDetails.cloudfront = identities.length === 0 ?
                    'None'
                    : identities.join(', ');
            })
                .catch((error) => {
                bucketDetails.cloudfront = error.message;
            })
                .finally(() => resolve());
        });
    }
}
exports.default = Csv;
